﻿using System.Drawing;

using DevExpress.XtraGrid.Columns;
using DevExpress.XtraGrid.Drawing;
using DevExpress.XtraGrid.Views.BandedGrid;
using DevExpress.XtraGrid.Views.Grid.Drawing;
using DevExpress.XtraGrid.Views.BandedGrid.Drawing;
using DevExpress.XtraGrid.Views.Grid.ViewInfo;

namespace TestProject
{
    /// <summary>
    /// DS 밴드 그리드 페인터
    /// </summary>
    public class DSBandedGridPainter : BandedGridPainter
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - DSBandedGridPainter(bandedGridView)

        /// <summary>
        /// 생성자
        /// </summary>
        /// <param name="bandedGridView">밴드 그리드 뷰</param>
        public DSBandedGridPainter(BandedGridView bandedGridView) : base(bandedGridView)
        {
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Protected

        #region 행 그리기 - DrawRows(e)

        /// <summary>
        /// 행 그리기
        /// </summary>
        /// <param name="e">이벤트 인자</param>
        protected override void DrawRows(GridViewDrawArgs e)
        {
            base.DrawRows(e);

            DrawEmptyAreaLines(e);
        }

        #endregion
        #region 비어있는 영역 선 그리기 - DrawEmptyAreaLines(e)

        /// <summary>
        /// 비어있는 영역 선 그리기
        /// </summary>
        /// <param name="e">그리드 뷰 그리기 인자</param>
        protected virtual void DrawEmptyAreaLines(GridViewDrawArgs e)
        {
            GridViewRects gridViewRects     = e.ViewInfo.ViewRects;
            Rectangle     emptyRowRectangle = gridViewRects.EmptyRows;

            if(emptyRowRectangle.IsEmpty)
            {
                return;
            }

            Pen pen = SystemPens.ControlDark;

            if(View.OptionsView.ShowVertLines)
            {
                foreach(GridColumnInfoArgs gridColumnInfoArgs in e.ViewInfo.ColumnsInfo)
                {
                    int x = gridColumnInfoArgs.Bounds.Right - 1;

                    if((gridColumnInfoArgs.Column != null && gridColumnInfoArgs.Column.Fixed != FixedStyle.None) || ((gridViewRects.FixedLeft.IsEmpty || x > gridViewRects.FixedLeft.Right) && (gridViewRects.FixedRight.IsEmpty || x < gridViewRects.FixedRight.Left - 3)))
                    {
                        e.Graphics.DrawLine(pen, x, emptyRowRectangle.Top, x, emptyRowRectangle.Bottom);
                    }
                }

                if(!gridViewRects.FixedRight.IsEmpty)
                {
                    e.Graphics.DrawLine(pen, gridViewRects.FixedRight.Left - 1, emptyRowRectangle.Top, gridViewRects.FixedRight.Left - 1, emptyRowRectangle.Bottom);
                }
            }

            if(View.OptionsView.ShowHorzLines)
            {
                int rowHeight = e.ViewInfo.MinRowHeight;

                for(int y = emptyRowRectangle.Top + rowHeight; y < emptyRowRectangle.Bottom; y += rowHeight)
                {
                    e.Graphics.DrawLine(pen, emptyRowRectangle.Left, y, gridViewRects.DataRectRight - 1, y);
                }
            }
        }

        #endregion
    }
}