﻿using System.Drawing;

using DevExpress.XtraGrid.Columns;
using DevExpress.XtraGrid.Drawing;
using DevExpress.XtraGrid.Views.Grid;
using DevExpress.XtraGrid.Views.Grid.Drawing;
using DevExpress.XtraGrid.Views.Grid.ViewInfo;

namespace TestProject
{
    /// <summary>
    /// DS 그리드 페인터
    /// </summary>
    public class DSGridPainter : GridPainter
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - DSGridPainter(gridView)

        /// <summary>
        /// 생성자
        /// </summary>
        /// <param name="gridView">그리드 뷰</param>
        public DSGridPainter(GridView gridView) : base(gridView)
        {
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Protected

        #region 정상 행 그리기 - DrawRegularRow(e, gridDataRowInfo)

        /// <summary>
        /// 정상 행 그리기
        /// </summary>
        /// <param name="e">이벤트 인자</param>
        /// <param name="gridDataRowInfo">그리드 데이타 행 정보</param>
        protected override void DrawRegularRow(GridViewDrawArgs e, GridDataRowInfo gridDataRowInfo)
        {
            base.DrawRegularRow(e, gridDataRowInfo);

            if(gridDataRowInfo.IsMasterRow && gridDataRowInfo.MasterRowExpanded)
            {
                GridView detailGridView = View.GetDetailView(gridDataRowInfo.RowHandle, 0) as GridView;

                if(detailGridView == null)
                {
                    return;
                }

                GridCellInfo gridCellInfo = gridDataRowInfo.Cells[View.VisibleColumns[0]];

                int center;

                if(gridCellInfo != null)
                {
                    center = gridCellInfo.CellButtonRect.Left + gridCellInfo.CellButtonRect.Width / 2;
                }
                else
                {
                    center = gridDataRowInfo.DetailIndentBounds.Left + gridDataRowInfo.DetailIndentBounds.Width / 2 ;
                }

                GridViewInfo gridViewInfo = detailGridView.GetViewInfo() as GridViewInfo;

                int   level = 0;
                Point startPoint;
                Point endPoint;

                foreach(GridRowInfo gridRowInfo in gridViewInfo.RowsInfo)
                {
                    if(detailGridView.IsRowVisible(gridRowInfo.RowHandle) != RowVisibleState.Visible)
                    {
                        continue;
                    }

                    level = gridRowInfo.Bounds.Top + gridRowInfo.Bounds.Height / 2;

                    startPoint = new Point(center, level);
                    endPoint   = new Point(gridDataRowInfo.DetailIndentBounds.Right, level);

                    e.Graphics.DrawLine(Pens.Black, startPoint, endPoint);
                }

                startPoint = new Point(center, gridDataRowInfo.DetailIndentBounds.Top);
                endPoint   = new Point(center, level);

                e.Graphics.DrawLine(Pens.Black, startPoint, endPoint);
            }
        }

        #endregion
        #region 행 그리기 - DrawRows(e)

        /// <summary>
        /// 행 그리기
        /// </summary>
        /// <param name="e">이벤트 인자</param>
        protected override void DrawRows(GridViewDrawArgs e)
        {
            base.DrawRows(e);

            DrawEmptyAreaLines(e);
        }

        #endregion
        #region 비어있는 영역 선 그리기 - DrawEmptyAreaLines(e)

        /// <summary>
        /// 비어있는 영역 선 그리기
        /// </summary>
        /// <param name="e">그리드 뷰 그리기 인자</param>
        protected virtual void DrawEmptyAreaLines(GridViewDrawArgs e)
        {
            GridViewRects gridViewRects     = e.ViewInfo.ViewRects;
            Rectangle     emptyRowRectangle = gridViewRects.EmptyRows;

            if(emptyRowRectangle.IsEmpty)
            {
                return;
            }

            Pen pen = SystemPens.ControlDark;

            if(View.OptionsView.ShowVertLines)
            {
                foreach(GridColumnInfoArgs gridColumnInfoArgs in e.ViewInfo.ColumnsInfo)
                {
                    int x = gridColumnInfoArgs.Bounds.Right - 1;

                    if((gridColumnInfoArgs.Column != null && gridColumnInfoArgs.Column.Fixed != FixedStyle.None) || ((gridViewRects.FixedLeft.IsEmpty || x > gridViewRects.FixedLeft.Right) && (gridViewRects.FixedRight.IsEmpty || x < gridViewRects.FixedRight.Left - 3)))
                    {
                        e.Graphics.DrawLine(pen, x, emptyRowRectangle.Top, x, emptyRowRectangle.Bottom);
                    }
                }

                if(!gridViewRects.FixedRight.IsEmpty)
                {
                    e.Graphics.DrawLine(pen, gridViewRects.FixedRight.Left - 1, emptyRowRectangle.Top, gridViewRects.FixedRight.Left - 1, emptyRowRectangle.Bottom);
                }
            }

            if(View.OptionsView.ShowHorzLines)
            {
                int rowHeight = e.ViewInfo.MinRowHeight;

                for(int y = emptyRowRectangle.Top + rowHeight; y < emptyRowRectangle.Bottom; y += rowHeight)
                {
                    e.Graphics.DrawLine(pen, emptyRowRectangle.Left, y, gridViewRects.DataRectRight - 1, y);
                }
            }
        }

        #endregion
    }
}