﻿using System.Drawing;

using DevExpress.Utils.Drawing;
using DevExpress.XtraGrid;
using DevExpress.XtraGrid.Columns;
using DevExpress.XtraGrid.Drawing;
using DevExpress.XtraGrid.Views.Grid;
using DevExpress.XtraGrid.Views.Printing;

namespace TestProject
{
    /// <summary>
    /// DS 그리드 뷰
    /// </summary>
    public class DSGridView : GridView
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Protected

        #region 뷰 명칭 - ViewName

        /// <summary>
        /// 뷰 명칭
        /// </summary>
        protected override string ViewName
        {
            get
            {
                return "DSGridView";
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - DSGridView()

        /// <summary>
        /// 생성자
        /// </summary>
        public DSGridView() : this(null)
        {
        }

        #endregion
        #region 생성자 - DSGridView(gridControl)

        /// <summary>
        /// 생성자
        /// </summary>
        /// <param name="gridControl">그리드 컨트롤</param>
        public DSGridView(GridControl gridControl) : base(gridControl)
        {
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method

        #region 커스텀 셀 그리기 인자 구하기 - GetCustomDrawCellArgs(graphicsCache, rectangle, gridColumn)

        /// <summary>
        /// 커스텀 셀 그리기 인자 구하기
        /// </summary>
        /// <param name="graphicsCache">그래픽스 캐시</param>
        /// <param name="rectangle">사각형</param>
        /// <param name="gridColumn">그리드 컬럼</param>
        /// <returns>꼬리말 셀 커스텀 그리기 이벤트 인자</returns>
        public FooterCellCustomDrawEventArgs GetCustomDrawCellArgs(GraphicsCache graphicsCache, Rectangle rectangle, GridColumn gridColumn)
        {
            GridFooterCellInfoArgs gridFooterCellInfoArgs = new GridFooterCellInfoArgs(graphicsCache);

            gridFooterCellInfoArgs.Bounds = new Rectangle(new Point(0, 0), rectangle.Size);

            FooterCellCustomDrawEventArgs e = new FooterCellCustomDrawEventArgs(graphicsCache, -1, gridColumn, null, gridFooterCellInfoArgs);

            e.Handled = true;

            RaiseCustomDrawFooterCell(e);

            return e;
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Protected

        #region 컬럼 컬렉션 생성하기 - CreateColumnCollection()

        /// <summary>
        /// 컬럼 컬렉션 생성하기
        /// </summary>
        /// <returns>그리드 컬럼 컬렉션</returns>
        protected override GridColumnCollection CreateColumnCollection()
        {
            return new DSGridColumnCollection(this);
        }

        #endregion
        #region 인쇄 정보 인스턴스 생성하기 - CreatePrintInfoInstance(e)

        /// <summary>
        /// 인쇄 정보 인스턴스 생성하기
        /// </summary>
        /// <param name="e">이벤트 인자</param>
        /// <returns>베이스 뷰 인쇄 정보</returns>
        protected override BaseViewPrintInfo CreatePrintInfoInstance(PrintInfoArgs e)
        {
            return new DSGridViewPrintInfo(e);
        }

        #endregion
    }
}