﻿using System.Drawing;

using DevExpress.Data;
using DevExpress.Utils.Drawing;
using DevExpress.XtraGrid.Views.Grid;
using DevExpress.XtraGrid.Views.Printing;
using DevExpress.XtraPrinting;

namespace TestProject
{
    /// <summary>
    /// DS 그리드 뷰 인쇄 정보
    /// </summary>
    public class DSGridViewPrintInfo : GridViewPrintInfo
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 꼬리말 패널 높이 - FooterPanelHeight

        /// <summary>
        /// 꼬리말 패널 높이
        /// </summary>
        public int FooterPanelHeight
        {
            get
            {
                return CalcStyleHeight(AppearancePrint.FooterPanel) + 4;
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - DSGridViewPrintInfo(printInfoArgs)

        /// <summary>
        /// 생성자
        /// </summary>
        /// <param name="printInfoArgs">인쇄 정보 인자</param>
        public DSGridViewPrintInfo(PrintInfoArgs printInfoArgs) : base(printInfoArgs)
        {
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 꼬리말 패널 인쇄하기 - PrintFooterPanel(brickGraphics)

        /// <summary>
        /// 꼬리말 패널 인쇄하기
        /// </summary>
        /// <param name="brickGraphics">브릭 그래픽스</param>
        public override void PrintFooterPanel(IBrickGraphics brickGraphics)
        {
            base.PrintFooterPanel(brickGraphics);

            CustomDrawFooterCells(brickGraphics);
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region 이미지 브릭 구하기 - GetImageBrick(printColumnInfo, rectangle, text)

        /// <summary>
        /// 이미지 브릭 구하기
        /// </summary>
        /// <param name="printColumnInfo">인쇄 컬럼 정보</param>
        /// <param name="rectangle">사각형</param>
        /// <param name="text">텍스트</param>
        /// <returns>이미지 브릭</returns>
        private ImageBrick GetImageBrick(PrintColumnInfo printColumnInfo, Rectangle rectangle, out string text)
        {
            Bitmap bitmap = new Bitmap(rectangle.Width, rectangle.Height);

            GraphicsCache graphicsCache = new GraphicsCache(Graphics.FromImage(bitmap));

            FooterCellCustomDrawEventArgs e = (View as DSGridView).GetCustomDrawCellArgs(graphicsCache, rectangle, printColumnInfo.Column);

            text = e.Info.DisplayText;

            if(!e.Handled)
            {
                return null;
            }

            BorderSide borderSide = e.Appearance.Options.UseBorderColor ? BorderSide.All : BorderSide.None;
            ImageBrick imageBrick = new ImageBrick(borderSide, 1, e.Appearance.BorderColor, e.Appearance.BackColor);

            imageBrick.Rect  = rectangle;
            imageBrick.Image = bitmap;

            return imageBrick;
        }

        #endregion
        #region 커스텀 꼬리말 셀 그리기 - CustomDrawFooterCells(brickGraphics)

        /// <summary>
        /// 커스텀 꼬리말 셀 그리기
        /// </summary>
        /// <param name="brickGraphics">브릭 그래픽스</param>
        private void CustomDrawFooterCells(IBrickGraphics brickGraphics)
        {
            if(!View.OptionsPrint.PrintFooter)
            {
                return;
            }

            foreach(PrintColumnInfo printColumnInfo in Columns)
            {
                if(printColumnInfo.Column.SummaryItem.SummaryType == SummaryItemType.None)
                {
                    continue;
                }

                Rectangle rectangle = Rectangle.Empty;

                rectangle.X      = printColumnInfo.Bounds.X + Indent;
                rectangle.Y      = printColumnInfo.RowIndex * FooterPanelHeight + 2 + Y;
                rectangle.Width  = printColumnInfo.Bounds.Width;
                rectangle.Height = FooterPanelHeight * printColumnInfo.RowCount;

                rectangle.X -= Indent;
                rectangle.Y -= rectangle.Height;

                string text = string.Empty;

                ImageBrick imageBrick = GetImageBrick(printColumnInfo, rectangle, out text);

                if(imageBrick != null)
                {
                    brickGraphics.DrawBrick(imageBrick, imageBrick.Rect);
                }
            }
        }

        #endregion
    }
}